// Dataflow von Vertex Shader zu Pixel Shader:
struct VertexToPixel
{
    float4 Position     : POSITION0;
    float2 TexCoords    : TEXCOORD0;
    float4 TexCoordHelper : TEXCOORD1;
};
// Dataflow von PixelShader -> Output
struct PixelToFrame
{
    float4 Color : COLOR0;
};

// Viewing & Projection Matrix
float4x4 xViewProjection;
float4x4 xWorld;
// Color-Coded Texturen vom Back und Frontface gecullten Wrfel:
Texture xFrontTexture;
Texture xBackTexture;
// Transfer Textur:
Texture xTranferTexture;
//??
float4 xViewport;

texture xVolumeTexture
< 
    string ResourceType = "VOLUME"; 
    //string function = "GenerateVolumeMap"; 
>;


// Sampler fr Volumstextur anlegen
sampler3D VolumeTexture =
sampler_state
{
    texture = (xVolumeTexture);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    
    AddressU = BORDER;
    AddressV = BORDER;
    AddressW = BORDER;
};

// Sampler fr Front und Back Textures
sampler FrontTextureSampler = 
sampler_state 
{ 
	texture = <xFrontTexture> ; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};
sampler BackTextureSampler = 
sampler_state 
{ 
	texture = <xBackTexture>; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};

// Sampler fr TransferFunktions-Textur
sampler TransferTextureSampler = 
sampler_state 
{ 
	texture = <xTranferTexture>; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};

// Vertex Shader:
VertexToPixel VertexMapper( float4 inPos : POSITION, float2 inTexCoords : TEXCOORD0)
{
    VertexToPixel Output;
    Matrix Mat = mul(xWorld, xViewProjection);
    Output.Position = mul(inPos, Mat);
    Output.TexCoords = inTexCoords;
    Output.TexCoordHelper.x = Output.Position.x;
    Output.TexCoordHelper.y = Output.Position.y;
    Output.TexCoordHelper.z = 0;
    Output.TexCoordHelper.w = Output.Position.w;

    return Output;
}

// Pixel Shader:
PixelToFrame PixelRayTracer(VertexToPixel PSIn)
{
    PixelToFrame Output;
    Output.Color.xyzw = 0.0;
    float4 ScreenCoordiantes;
    ScreenCoordiantes.x = (PSIn.TexCoordHelper.x /(PSIn.TexCoordHelper.w * 2)) + 0.5f;// * xViewport.x;
    ScreenCoordiantes.y = (PSIn.TexCoordHelper.y /(PSIn.TexCoordHelper.w * 2)) - 0.5f;// * xViewport.y;
    ScreenCoordiantes.zw = 0.0;
 
    float4 entryPoint = tex2Dlod(FrontTextureSampler, ScreenCoordiantes );
    float4 exitPoint = tex2Dlod(BackTextureSampler, ScreenCoordiantes );
    
    float dist = distance(entryPoint, exitPoint);

	float4 diff = normalize(exitPoint - entryPoint) / 200;
	float4 samplePoint = entryPoint;
	samplePoint.w = 0.0;

	int i = 0;
	
	while (i < 250)
	{
		float4 intensity = tex3Dlod(VolumeTexture, samplePoint) * 16;
		float2 transferlookup;
		transferlookup.y = intensity.x;
		transferlookup.x = 1.0;
		
		float4 transfered = tex2D(TransferTextureSampler, transferlookup);

		if (intensity.x > Output.Color.w)
		{
			Output.Color.xyz = intensity.x * transfered.xyz;

			Output.Color.w = intensity.x;
		}
		
		samplePoint += diff;
		i++;
	}
    return Output;
}


technique RayTracer
{
    pass Pass0
    {        
        VertexShader = compile vs_3_0 VertexMapper();
        PixelShader = compile ps_3_0 PixelRayTracer();
    }
}