// Dataflow von Vertex Shader zu Pixel Shader:
struct VertexToPixel
{
    float4 Position     : POSITION0;
    float2 TexCoords    : TEXCOORD0;
    float4 TexCoordHelper : TEXCOORD1;
    float4 LightDir : TEXCOORD2;
};
// Dataflow von PixelShader -> Output
struct PixelToFrame
{
    float4 Color : COLOR0;
};

// Viewing & Projection Matrix
float4x4 xViewProjection;
float4x4 xWorld;
// Color-Coded Texturen vom Back und Frontface gecullten Wrfel:
Texture xFrontTexture;
Texture xBackTexture;
// Transfer Textur:
Texture xTranferTexture;
//??
float4 xViewport;
// sampling-stepsize:
float xkA, xkD, xkS, xGradDep;
int xn;
int xStepSize;
int xDataSize;
// Volume texture

float xLightPos;
texture xVolumeTexture
< 
    string ResourceType = "VOLUME"; 
    //string function = "GenerateVolumeMap"; 
>;


// Sampler fr Volumstextur anlegen
sampler3D VolumeTexture =
sampler_state
{
    texture = (xVolumeTexture);
    MipFilter = LINEAR;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    
    AddressU = BORDER;
    AddressV = BORDER;
    AddressW = BORDER;
};

// Sampler fr Front und Back Textures
sampler FrontTextureSampler = 
sampler_state 
{ 
	texture = <xFrontTexture> ; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};
sampler BackTextureSampler = 
sampler_state 
{ 
	texture = <xBackTexture>; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};

// Sampler fr TransferFunktions-Textur
sampler TransferTextureSampler = 
sampler_state 
{ 
	texture = <xTranferTexture>; 
	magfilter = LINEAR; 
	minfilter = LINEAR; 
	mipfilter=LINEAR; 
	AddressU = mirror; 
	AddressV = mirror;
};

// Vertex Shader:
VertexToPixel VertexMapper( float4 inPos : POSITION, float2 inTexCoords : TEXCOORD0)
{
    VertexToPixel Output;
    Matrix Mat = mul(xWorld, xViewProjection);
    Output.Position = mul(inPos, Mat);
    Output.TexCoords = inTexCoords;
    Output.TexCoordHelper.x = Output.Position.x;
    Output.TexCoordHelper.y = Output.Position.y;
    Output.TexCoordHelper.z = 0;
    Output.TexCoordHelper.w = Output.Position.w;
    
    float4 LightPos = float4(10,0,0,0); 
    
    float4 LightDir = normalize(LightPos - inPos);
    
    Output.LightDir = LightDir;

    return Output;
}

// Pixel Shader:
PixelToFrame PixelRayTracer(VertexToPixel PSIn)
{
    PixelToFrame Output;
    Output.Color.xyzw = 0.0;
    float4 ScreenCoordiantes;
    ScreenCoordiantes.x = (PSIn.TexCoordHelper.x /(PSIn.TexCoordHelper.w * 2)) + 0.5f;// * xViewport.x;
    ScreenCoordiantes.y = (PSIn.TexCoordHelper.y /(PSIn.TexCoordHelper.w * 2)) - 0.5f;// * xViewport.y;
    ScreenCoordiantes.zw = 0.0;
 
    float4 entryPoint = tex2Dlod(FrontTextureSampler, ScreenCoordiantes );
    float4 exitPoint = tex2Dlod(BackTextureSampler, ScreenCoordiantes );
    float dist = distance(entryPoint.xyz, exitPoint.xyz);
	
	float4 diff;
	diff.xyz = normalize(exitPoint.xyz - entryPoint.xyz) / xStepSize;
	diff.w = 0.0;
	
	float4 samplePoint = entryPoint;
	samplePoint.w = 0.0;
    

	float3 lightVec = float3(10,0,0);
	lightVec = normalize(lightVec);
	float3 ViewVec = normalize(float3(0,0,10));
	float3 h = normalize(lightVec + ViewVec);
	
	bool work = true;
	int i = 0;
	int max_iter = int(ceil(dist/length(diff.xyz)));

	while(i < max_iter && Output.Color.w < 1.0)
	{
		float4 intensity = tex3Dlod(VolumeTexture, samplePoint) * 16;
		float3 gradient; 

		float4 gr_p1;				
		gr_p1 = tex3Dlod(VolumeTexture, float4(samplePoint.x + float(1)/(float)xDataSize, samplePoint.y, samplePoint.z, 0))* 16 - tex3Dlod(VolumeTexture, float4(samplePoint.x - float(1)/(float)xDataSize, samplePoint.y, samplePoint.z, 0))* 16;
			
		float4 gr_p2;
		gr_p2 = tex3Dlod(VolumeTexture, float4(samplePoint.x, samplePoint.y + float(1)/(float)xDataSize, samplePoint.z, 0))* 16 - tex3Dlod(VolumeTexture, float4(samplePoint.x, samplePoint.y - float(1)/(float)xDataSize, samplePoint.z, 0))* 16;
			
		float4 gr_p3;
		gr_p3 = tex3Dlod(VolumeTexture, float4(samplePoint.x, samplePoint.y, samplePoint.z + float(1)/(float)xDataSize, 0))* 16 - tex3Dlod(VolumeTexture, float4(samplePoint.x, samplePoint.y, samplePoint.z - float(1)/(float)xDataSize, 0))* 16;
			
		gradient.x = gr_p1.x/2;
		gradient.y = gr_p2.x/2;
		gradient.z = gr_p3.x/2;				
		float gradlen = length(gradient);
		Matrix Mat = mul(xWorld, xViewProjection);
        gradient = mul(gradient, Mat);

		gradient = normalize(gradient);
		float4 transferlookup;
		transferlookup.y = intensity.x;
		transferlookup.x =  ((xGradDep * (gradlen / intensity.x)) + (float(1) - xGradDep));
		transferlookup.zw = 0.0;
		float4 transfered = tex2Dlod(TransferTextureSampler, transferlookup);

		float3 col = 0;

		col.xyz = saturate(xkA * transfered.xyz + xkD * transfered.xyz * dot(gradient,lightVec)+ xkS * pow(dot(gradient,h),xn));

		Output.Color.xyz += (1.0 - Output.Color.w) *transfered.w * col.xyz;

		Output.Color.w += (1.0 - Output.Color.w) * transfered.w;
		
		samplePoint.xyz += diff.xyz;
		
		i++;
	}
	
    return Output;
}


technique RayTracer
{
    pass Pass0
    {        
        VertexShader = compile vs_3_0 VertexMapper();
        PixelShader = compile ps_3_0 PixelRayTracer();
    }
}